package com.flowingdata.gps
{
	import com.modestmaps.Map;
	import com.modestmaps.events.MapEvent;
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.text.TextField;
	import flash.text.TextFormat;
	

	public class MarkersClip extends Sprite
	{
		protected var map:Map;
		public var markers:Array;
		public var isStationary:Boolean;
		private var color:uint;
		
		public var locations:Array;
		private var openingDates:Array;
		
		private var storesPerYear:Array = new Array();
		public var currentYear:String;
		private var storesPerYearIndex:Number;
		private var currentYearCount:Number = 0;
		private var currentRate:Number;
				
		private var xpoints:Array = new Array();
		private var ypoints:Array = new Array();
		
		public var markerIndex:Number = 0;
		private var starting:Point;
		
		private var pause:Boolean = false;
		
		private var yearLabel:TextField;
		private var font:TextFormat;
		
		// enable this if you want intermediate zooming steps to
        // stretch your graphics instead of reprojecting the points
        // it looks worse and probably isn't faster, but there it is :)
        public var scaleZoom:Boolean = false;
		
		
		public function MarkersClip(map:Map, locations:Array, openingDates:Array, color:uint) 
		{	
			this.map = map;
			this.color = color;
			
			this.x = map.getWidth() / 2;
			this.y = map.getHeight() / 2;
			
			this.locations = locations;
			setPoints();
			setMarkers();
			
			this.openingDates = openingDates;
			
			var tempIndex:int = 0;			
			
			storesPerYear = [1,1,1,2,5,5,5,15,17,19,25,19,27,39,34,43,54,150,63,87,99,110,121,142,125,131,178,163,138,156,107,129,53,60,66,80,105,106,114,96,130,118,37];
			
			// Reset
			storesPerYearIndex = 0;
			currentYear = openingDates[markerIndex];
			
			
			this.addEventListener(Event.ENTER_FRAME, onEnterFrame);
			this.map.addEventListener(MapEvent.START_ZOOMING, onMapStartZooming);
	        this.map.addEventListener(MapEvent.STOP_ZOOMING, onMapStopZooming);
	        this.map.addEventListener(MapEvent.ZOOMED_BY, onMapZoomedBy);
			this.map.addEventListener(MapEvent.START_PANNING, onMapStartPanning);
	        this.map.addEventListener(MapEvent.STOP_PANNING, onMapStopPanning);
	        this.map.addEventListener(MapEvent.PANNED, onMapPanned);
		}
		
		
		protected function setMarkers():void
		{
			markers = new Array();
			
			for (var i:int = 0; i < locations.length; i++) 
			{
				var marker:Marker = new Marker();
				addChild(marker);
				marker.x = xpoints[i]; marker.y = ypoints[i];
				markers.push(marker);
			}
			
		}
		
		
		public function setPoints():void {
			if (locations == null) {
				return;
			}			
			
			var p:Point;
			for (var i:int = 0; i < locations.length; i++) {			
				p = map.locationPoint(locations[i], this);
				xpoints[i] = p.x;
				ypoints[i] = p.y;
			}			
		}
		
		
		public function updatePoints():void 
		{	
			setPoints();
			
			for (var i:int = 0; i < markers.length; i++) {
				Marker(markers[i]).x = xpoints[i]; Marker(markers[i]).y = ypoints[i];
			}
			
		}
		
		
		public function destroy():void {
			
			map.removeEventListener(MapEvent.START_ZOOMING, onMapStartZooming);
	        map.removeEventListener(MapEvent.STOP_ZOOMING, onMapStopZooming);
	        map.removeEventListener(MapEvent.ZOOMED_BY, onMapZoomedBy);
			map.removeEventListener(MapEvent.START_PANNING, onMapStartPanning);
	        map.removeEventListener(MapEvent.STOP_PANNING, onMapStopPanning);
	        map.removeEventListener(MapEvent.PANNED, onMapPanned);
	        map.removeEventListener(MapEvent.EXTENT_CHANGED, onExtentChanged);
	        map.removeEventListener(MapEvent.RESIZED, onMapResized);
		}
		
		
		
		/* @ Listeners */
		
		private function onEnterFrame(event:Event):void
		{
			
			
			// If haven't gone through all the stores or panning or zooming:
			if (markerIndex < markers.length && !pause) 
			{	
				// If played all stores for current year:
				if (currentYearCount > storesPerYear[storesPerYearIndex]) {
					
					// Move on to next year
					storesPerYearIndex++;
					currentYearCount = 0;
				}
				currentRate = storesPerYear[storesPerYearIndex] / stage.frameRate;
				
				//currentRate = 20;
				
				var playCount:Number;
				if (currentRate >= 1) {
					playCount = Math.floor(currentRate);
				} 
				
				else {
					var playCountProb:Number = currentRate;
					if (Math.random() <= playCountProb) 
						playCount = 1;
					else
						playCount = 0;
				}
				
				
				
				for (var i:int = 0; i < playCount; i++) {
					
					if (markerIndex >= markers.length) break;
					
					Marker(markers[markerIndex]).play();
					
					markerIndex++;				
					currentYearCount++;
				}
				

				currentYear = String(openingDates[markerIndex]);
			}
		}
		
		
		private function onMapStartZooming(event:MapEvent):void
	    {
	    	pause = true;
	        for (var i:int = 0; i < markers.length; i++) {
	        	Marker(markers[i]).clear();
	        }
	    }
	    
	    private function onMapStopZooming(event:MapEvent):void
	    {
	    	pause = false;
	        if (scaleZoom) {
	            scaleX = scaleY = 1.0;
	        }
            updatePoints();
            for (var i:int = 0; i < markers.length; i++) {
	        		Marker(markers[i]).draw();
	        }
	    }
	    
	    private function onMapZoomedBy(event:MapEvent):void
	    {
	        if (scaleZoom) {
    	        scaleX = scaleY = Math.pow(2, event.zoomDelta);
	        }
	        else {
                
	        }
	    }
		
		
		
		private function onMapStartPanning(event:MapEvent):void
	    {
	    	pause = true;
	        starting = new Point(x, y);
	        trace('xpoint on start ' + xpoints[0]);
	    }
		
		
		private function onMapPanned(event:MapEvent):void
	    {
	        if (starting) {
	            x = starting.x + event.panDelta.x;
	            y = starting.y + event.panDelta.y;
	        }
	        else {
	            x = event.panDelta.x;
	            y = event.panDelta.y;	            
	        }
	    }
	    
	    
	    private function onMapStopPanning(event:MapEvent):void
	    {
			pause = false;
	    }
	    
	    
	    private function onExtentChanged(event:MapEvent):void
		{
			updatePoints();
		}
	    
	    
	    private function onMapResized(event:MapEvent):void
	    {
	        x = event.newSize[0]/2;
	        y = event.newSize[1]/2;
	        updatePoints();
	    }
		
		
	}
}